From eeb92c719f86389114fcb5c8cb00fcf40c0f36da Mon Sep 17 00:00:00 2001
From: Roberto Corno <corno.roberto@gmail.com>
Date: Tue, 22 May 2012 10:56:29 +0200
Subject: [PATCH 2/3] Add 0x41 descriptor support

---
 src/Makefile.am         |    2 +
 src/descriptors/dr.h    |    1 +
 src/descriptors/dr_41.c |  121 +++++++++++++++++++++++++++++++++++++++++++++++
 src/descriptors/dr_41.h |  104 ++++++++++++++++++++++++++++++++++++++++
 4 files changed, 228 insertions(+), 0 deletions(-)
 create mode 100644 src/descriptors/dr_41.c
 create mode 100644 src/descriptors/dr_41.h

diff --git a/src/Makefile.am b/src/Makefile.am
index cda6fae..795dcb8 100644
--- a/src/Makefile.am
+++ b/src/Makefile.am
@@ -32,6 +32,7 @@ pkginclude_HEADERS = dvbpsi.h psi.h descriptor.h demux.h custom.h \
                      descriptors/dr_0e.h \
                      descriptors/dr_0f.h \
                      descriptors/dr_40.h
+                     descriptors/dr_41.h \
                      descriptors/dr_42.h \
                      descriptors/dr_43.h \
                      descriptors/dr_44.h \
@@ -65,6 +66,7 @@ descriptors_src = descriptors/dr_02.c \
                   descriptors/dr_0e.c \
                   descriptors/dr_0f.c \
                   descriptors/dr_40.c
+                  descriptors/dr_41.c \
                   descriptors/dr_42.c \
                   descriptors/dr_43.c \
                   descriptors/dr_44.c \
diff --git a/src/descriptors/dr.h b/src/descriptors/dr.h
index 93b27ee..5e71541 100644
--- a/src/descriptors/dr.h
+++ b/src/descriptors/dr.h
@@ -47,6 +47,7 @@
 #include "dr_0e.h"
 #include "dr_0f.h"
 #include "dr_40.h"
+#include "dr_41.h"
 #include "dr_42.h"
 #include "dr_43.h"
 #include "dr_44.h"
diff --git a/src/descriptors/dr_41.c b/src/descriptors/dr_41.c
new file mode 100644
index 0000000..48fb582
--- /dev/null
+++ b/src/descriptors/dr_41.c
@@ -0,0 +1,121 @@
+/*
+ * dr_41.c
+ * Copyright (C) 2012 VideoLAN
+ *
+ * Authors: rcorno (May 21, 2012)
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ *****************************************************************************/
+
+#include "config.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <stdbool.h>
+#include <string.h>
+
+#if defined(HAVE_INTTYPES_H)
+#include <inttypes.h>
+#elif defined(HAVE_STDINT_H)
+#include <stdint.h>
+#endif
+
+#include "../dvbpsi.h"
+#include "../dvbpsi_private.h"
+#include "../descriptor.h"
+
+#include "dr_41.h"
+
+/*****************************************************************************
+ * dvbpsi_DecodeServiceListDr
+ *****************************************************************************/
+dvbpsi_service_list_dr_t* dvbpsi_DecodeServiceListDr(
+                                        dvbpsi_descriptor_t * p_descriptor)
+{
+    dvbpsi_service_list_dr_t * p_decoded;
+    int i,service_count;
+
+    /* Check the tag */
+    if (p_descriptor->i_tag != 0x41)
+        return NULL;
+
+    /* Don't decode twice */
+    if (p_descriptor->p_decoded)
+        return p_descriptor->p_decoded;
+
+    /* Check the length */
+    service_count = p_descriptor->i_length / 3;
+    if((p_descriptor->i_length < 1) || (p_descriptor->i_length % 3 != 0) || (service_count>63))
+      return NULL;
+
+    /* Allocate memory */
+    p_decoded = (dvbpsi_service_list_dr_t*)calloc(1, sizeof(dvbpsi_service_list_dr_t));
+    if (!p_decoded)
+        return NULL;    
+	
+    /* Decode data */    
+    p_decoded->i_service_count = service_count;
+    i=0;
+    while( i < p_decoded->i_service_count ) {
+    	p_decoded->i_service[i].i_service_id = ((uint16_t)(p_descriptor->p_data[i*3]) << 8)
+                                      | p_descriptor->p_data[i*3+1];
+    	p_decoded->i_service[i].i_service_type = p_descriptor->p_data[i*3+2];
+    	i++;
+    }
+
+    p_descriptor->p_decoded = (void*)p_decoded;
+
+    return p_decoded;
+}
+
+
+/*****************************************************************************
+ * dvbpsi_GenServiceListDr
+ *****************************************************************************/
+dvbpsi_descriptor_t * dvbpsi_GenServiceListDr(
+		                        dvbpsi_service_list_dr_t * p_decoded,
+                                        bool b_duplicate)
+{
+    /* Check the length */    
+    if (p_decoded->i_service_count > 63)
+        return NULL;
+
+    /* Create the descriptor */
+    dvbpsi_descriptor_t * p_descriptor =
+            dvbpsi_NewDescriptor(0x83, p_decoded->i_service_count*3, NULL);
+
+    if (!p_descriptor)
+        return NULL;
+
+    /* Encode data */
+    int i = 0;
+    while( i < p_decoded->i_service_count ) {
+      p_descriptor->p_data[i*3] = p_decoded->i_service[i].i_service_id >> 8;
+      p_descriptor->p_data[i*3+1] = p_decoded->i_service[i].i_service_id;
+      p_descriptor->p_data[i*3+2] = p_decoded->i_service[i].i_service_type;
+      i++;
+    }
+
+    if (b_duplicate)
+    {
+        /* Duplicate decoded data */
+      p_descriptor->p_decoded =
+               dvbpsi_DuplicateDecodedDescriptor(p_descriptor->p_decoded,
+                       sizeof(dvbpsi_service_list_dr_t));
+    }
+
+    return p_descriptor;
+}
diff --git a/src/descriptors/dr_41.h b/src/descriptors/dr_41.h
new file mode 100644
index 0000000..dc7b171
--- /dev/null
+++ b/src/descriptors/dr_41.h
@@ -0,0 +1,104 @@
+/*****************************************************************************
+ * dr_41.h
+ * Copyright (C) 2012 VideoLAN
+ *
+ * Authors: rcorno (May 21, 2012)
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ *****************************************************************************/
+
+/*!
+ * \file <dr_41.h>
+ * \author Corno Roberto <corno.roberto@gmail.com>
+ * \brief Application interface for the DVB "service list"
+ * descriptor decoder and generator.
+ *
+ * Application interface for the DVB "service list" descriptor
+ * decoder and generator. This descriptor's definition can be found in
+ * ETSI EN 300 468 section 6.2.35.
+ */
+
+#ifndef DR_41_H_
+#define DR_41_H_
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/*****************************************************************************
+ * dvbpsi_network_name_dr_t
+ *****************************************************************************/
+/*!
+ * \struct dvbpsi_service_list_dr_t
+ * \brief "service list" descriptor structure.
+ *
+ * This structure is used to store a decoded "service list"
+ * descriptor. (ETSI EN 300 468 section 6.2.35).
+ */
+/*!
+ * \typedef struct dvbpsi_service_list_dr_s dvbpsi_service_list_dr_t
+ * \brief dvbpsi_service_list_dr_t type definition.
+ */
+typedef struct dvbpsi_service_list_dr_s
+{
+  uint8_t       i_service_count;            /*!< length of the i_service_list
+  	                                             array */
+  struct {
+      uint16_t     i_service_id;            /*!< service id */
+      uint8_t      i_service_type;          /*!< service type */
+  } i_service[64];
+
+} dvbpsi_service_list_dr_t;
+
+/*****************************************************************************
+ * dvbpsi_DecodeServiceListDr
+ *****************************************************************************/
+/*!
+ * \fn dvbpsi_service_list_dr_t * dvbpsi_DecodeServiceListDr(
+                                        dvbpsi_descriptor_t * p_descriptor)
+ * \brief "service list" descriptor decoder.
+ * \param p_descriptor pointer to the descriptor structure
+ * \return a pointer to a new "service list" descriptor structure
+ * which contains the decoded data.
+ */
+dvbpsi_service_list_dr_t* dvbpsi_DecodeServiceListDr(
+                                        dvbpsi_descriptor_t * p_descriptor);
+
+
+/*****************************************************************************
+ * dvbpsi_GenServiceListDr
+ *****************************************************************************/
+/*!
+ * \fn dvbpsi_descriptor_t * dvbpsi_GenServiceListDr(
+                        dvbpsi_service_list_dr_t * p_decoded, int b_duplicate)
+ * \brief "service list" descriptor generator.
+ * \param p_decoded pointer to a decoded "service list" descriptor
+ * structure
+ * \param b_duplicate if non zero then duplicate the p_decoded structure into
+ * the descriptor
+ * \return a pointer to a new descriptor structure which contains encoded data.
+ */
+dvbpsi_descriptor_t * dvbpsi_GenServiceListDr(
+		                        dvbpsi_service_list_dr_t * p_decoded,
+                                        bool b_duplicate);
+
+#ifdef __cplusplus
+};
+#endif
+
+#else
+#error "Multiple inclusions of dr_41.h"
+#endif /* DR_41_H_ */
-- 
1.7.5.4
